﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Threading;
namespace OFSetter
{

    [StructLayout(LayoutKind.Sequential)]
    public struct POINT
    {
        public int x;
        public int y;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct RECT
    {
        public int left;
        public int top;
        public int right;
        public int bottom;
    }

    public partial class MainForm : Form
    {
        private bool targetting;
        private IntPtr hWndTarget;
		private uint timeWndSwitch;
		private uint timeChar;


        [DllImport("user32.dll")]
        private static extern IntPtr WindowFromPoint(POINT Point);

        [DllImport("user32.dll")]
        private static extern IntPtr GetAncestor(IntPtr hwnd, uint gaFlags);

        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        private static extern int GetWindowText(IntPtr hWnd, StringBuilder lpString, int nMaxCount);

		[DllImport("user32.dll")]
        private static extern bool SetForegroundWindow(IntPtr hWnd);

		[DllImport("user32.dll")]
        private static extern IntPtr GetForegroundWindow();
		private const uint GA_ROOT = 2;

		public MainForm()
        {
            InitializeComponent();
            targetting = false;
            hWndTarget = new IntPtr(0);
        }

        private void button1_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left) {
                this.Capture = true;
                targetting = true;
            }
        }

        private void Form1_MouseMove(object sender, MouseEventArgs e)
        {
            if (targetting) {
                Point pt = new Point(e.X, e.Y);
                pt = this.PointToScreen(pt);

                System.Diagnostics.Debug.WriteLine("Mouse pos x:" + pt.X + " y:" + pt.Y);
            }
        }

        private void Form1_MouseUp(object sender, MouseEventArgs e)
        {
            if (targetting) {
                Point pt = new Point(e.X, e.Y);
                pt = this.PointToScreen(pt);

                System.Diagnostics.Debug.WriteLine("Mouse pos x:" + pt.X + " y:" + pt.Y);
                targetting = false;

                POINT ptMouseup;
                ptMouseup.x = pt.X;
                ptMouseup.y = pt.Y;
                IntPtr hWnd0 = WindowFromPoint(ptMouseup);
                if (hWnd0 == IntPtr.Zero) {
                    System.Diagnostics.Debug.WriteLine("No Window was found.");
                    return;
                }
                hWndTarget = GetAncestor(hWnd0, GA_ROOT);

                StringBuilder sb = new StringBuilder(1024);
				if (hWndTarget == IntPtr.Zero) {
                    System.Diagnostics.Debug.WriteLine("Error occurs in finding the root window.");
                    return;
                }
                GetWindowText(hWndTarget, sb, sb.Capacity);
                textBoxTargetTitle.Text = sb.ToString();
                buttonTfer.Enabled = true;

				////Sendkeysの動作の確認

				//if (!SetForegroundWindow(hWndTarget)) {
				//    NoTagetWindowErr();
				//    return;
				//}
				//int timeCounter = 0;
				//while (GetForegroundWindow() != hWndTarget) {
				//    System.Threading.Thread.Sleep(100);
				//    timeCounter++;
				//    if (timeCounter > 50) {
				//        MessageBox.Show("ウィンドウの切り換えでタイムアウトが発生しました", "OFSetter", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1);
				//        return;
				//    }
				//}
				//System.Threading.Thread.Sleep(500);
				//SendKeys.Flush();
				//SendKeys.SendWait("\t");
				//System.Threading.Thread.Sleep(500);
				//SendKeys.Flush();
				//SendKeys.SendWait("\t");

            }    
        }

        private void buttonExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void NoTagetWindowErr()
        {
            MessageBox.Show("転送先ウィンドウが見つかりません","OFSetter",MessageBoxButtons.OK,MessageBoxIcon.Error,MessageBoxDefaultButton.Button1);
            hWndTarget = new IntPtr(0);
            textBoxTargetTitle.Text = "";
            buttonTfer.Enabled = false;
        }

        private void SendData(int column, int lines)
        {
            StringReader sr = new StringReader(textTferData.Text);
            while (lines > 0) {
                String str = sr.ReadLine();
                int currentColumn = 0;
                lines--;
                if(str == null){
                    break;
                }
                int cc = str.Length;
                for (int i = 0; i < cc; i++) {
                    if (GetForegroundWindow() != hWndTarget) {
                        MessageBox.Show("指定した転送先以外のウィンドウがアクティブになりました\n処理を中断します", "OFSetter", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1);
                        lines = 0;
                        break;
                    }
                    //タブが見つかったらカラムを進める
                    if (str[i] == '\t') {
                        currentColumn++;
                    }
                    //転送カラム数を超えたら中断する。
                    if (currentColumn >= column) {
                        break;
                    }
					System.Threading.Thread.Sleep((int)timeChar);
					SendKeys.Flush();
                    switch (str[i]) {
                        case '\t':
							SendKeys.SendWait("{TAB}"); 
                            break;
                        case '+':
                        case '%':
                        case '~':
                        case '(':
                        case ')':
                        case '{':
                        case '}':
                        case '[':
                        case ']':
							SendKeys.SendWait("{" + str[i].ToString() + "}");	
                            break;
                        default:
							SendKeys.SendWait(str[i].ToString());
                            break;
                    }
                }
                //たいていの入力フォームで行の終わりでタブを打つと次の行の先頭に移動する
				System.Threading.Thread.Sleep((int)timeChar);
				SendKeys.Flush();
				SendKeys.SendWait("{TAB}"); 

            }
            textTferData.Text = sr.ReadToEnd();
        }


        private void buttonTfer_Click(object sender, EventArgs e)
        {


            int column;
            int maxTferLine;


            column = GetColumn();
            textColumn.Text = column.ToString();


            maxTferLine = GetTferLine();
            textTferLine.Text = maxTferLine.ToString();


            if (!SetForegroundWindow(hWndTarget)) {
                NoTagetWindowErr();
                return;
            }

            int timeCounter = 0;
            while (GetForegroundWindow() != hWndTarget) {
                System.Threading.Thread.Sleep(100);
                timeCounter++;
                if (timeCounter > 50) {
                    MessageBox.Show("ウィンドウの切り換えでタイムアウトが発生しました", "OFSetter", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1);
                    return;
                }
            }

			System.Threading.Thread.Sleep((int)timeWndSwitch);
                

            SendData(column, maxTferLine);

            //残りデータ長が0なら、転送ボタンをディスエーブルする。
            if (textTferData.Text.Length == 0) {
                buttonTfer.Enabled = false;
            }
            

        }

        
        /// <summary>
        /// 転送行数の取得
        /// </summary>
        /// <returns></returns>
        private int GetTferLine()
        {
            int maxTferLine;
            try {
                maxTferLine = Convert.ToInt32(textTferLine.Text);
                if (maxTferLine < 1) {
                    maxTferLine = 1;
                } else if (maxTferLine > 50) {
                    maxTferLine = 50;
                }
            }
            catch {
                maxTferLine = 1;
            }
            return maxTferLine;
        }

        /// <summary>
        ///列数の取得
        /// </summary>
        /// <returns></returns>
        private int GetColumn()
        {
            int column;
            try {
                column = Convert.ToInt32(textColumn.Text);
                if (column < 1) {
                    column = 1;
                } else if (column > 10) {
                    column = 10;
                }
            }
            catch {
                column = 1;
            }
            return column;
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            uint n;
            n = Properties.Settings.Default.TferColumn;
            textColumn.Text =  n.ToString();
            n=Properties.Settings.Default.TferLine;
            textTferLine.Text = n.ToString();

			n = Properties.Settings.Default.TimeWndSwitch;
			if (n < 50) n = 50;
			else if (n > 2000) n = 2000;
			timeWndSwitch = n;

			n = Properties.Settings.Default.TimeChar;
			if (n < 50) n = 50;
			else if (n > 2000) n = 2000;
			timeChar = n;
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            int n;

            n = GetColumn();
            Properties.Settings.Default.TferColumn = (uint)n;

            n = GetTferLine();
            Properties.Settings.Default.TferLine = (uint)n;

            Properties.Settings.Default.Save();
        }

		private void buttonOption_Click(object sender, EventArgs e)
		{
			SetOption dlg = new SetOption();
			dlg.timeWndSwitch = timeWndSwitch;
			dlg.timeChar = timeChar;
			if (dlg.ShowDialog() == DialogResult.OK) {
				timeWndSwitch = dlg.timeWndSwitch;
				Properties.Settings.Default.TimeWndSwitch = timeWndSwitch;
				timeChar = dlg.timeChar;
				Properties.Settings.Default.TimeChar = timeChar;
			}
		
		}

    }
}
